package hk.advanpro.android.core.log;

import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import hk.advanpro.android.core.AdvanproCore;
import hk.advanpro.android.core.utils.DateUtils;
import hk.advanpro.android.core.utils.FileUtils;
import hk.advanpro.android.core.utils.SharedPreferencesUtils;
import hk.advanpro.android.core.utils.SystemUtils;
import hk.advanpro.android.core.utils.UIUtils;

/**
 * SDK在使用过程中会记录文件日志，方便出现问题时进行查错、调试。
 * 日志按日期分目录，按固定行数分文件存储，以当天零点的时间戳和文件号命名，
 * 日志默认最多保留7天，也可以通过参数进行设置。
 * 日志记录路径为：/logs/yyyyMMdd/platform-timestamp-num.log
 * yyyyMMdd : 日期
 * platform : 平台，ios、android
 * timestamp : 当天零点时间戳（毫秒）
 * num : 文件号，从1开始递增，默认按5万行划分。
 * Created by jalin on 2017/12/7.
 */

public class ALogger {
    public static final String TAG = "ALogger";

    public static final String LOG_FILE_DIR = "/logs";

    public static final String TYPE_INFO = "I";
    public static final String TYPE_DEBUG = "D";// 调试类型，非 debug 模式下不写入文件
    public static final String TYPE_ERROR = "E";

    public static final String KEY_START = "st";
    public static final String KEY_USER_ID = "uid";
    public static final String KEY_ERROR = "err";
    public static final String KEY_URL = "url";
    public static final String KEY_ACCOUNT = "account";
    public static final String KEY_TYPE = "ty";

    public static final String SP_KEY_LOG_TIMESTAMP = "hk.advanpro.android.core.LOG_TIMESTAMP";
    public static final String SP_KEY_LOG_LINE_COUNT = "hk.advanpro.android.core.LOG_LINE_COUNT";

    // 日志保留日按时，默认7天，超过此数量将删除旧的日志文件
    private int logRetainCount = 7;
    //日志行数，默认61440行，超过该行数自动分文件记录，文件号从1开始
    private int maxLineCount = 61440;
    private static ALogger instance;

    private FileUtils fileUtils;
    private SharedPreferencesUtils preferencesUtils;
    private long timestamp = 0;//日期0点整
    private int lineCount = 0;//行号
    private String fileDirPath;//文件目录路径，格式/Android/包名/file/Logs/yyyyMMdd

    private Object lock = new Object();

    public static ALogger getInstance() {
        if (instance == null) {
            synchronized (TYPE_INFO) {
                instance = new ALogger();
            }
        }
        return instance;
    }

    public void setMaxLineCount(int maxLineCount) {
        this.maxLineCount = maxLineCount;
    }

    public void setLogRetainCount(int logRetainCount) {
        this.logRetainCount = logRetainCount;
    }

    private ALogger() {
        if (AdvanproCore.getAppContext() == null) {
            throw new NullPointerException(
                    "'hk.advanpro.android.core.AdvanproCore.getAppContext()' on a null object reference," +
                            "please init AdvanproCore, 'hk.advanpro.android.core.AdvanproCore.initWithConfig(AdvanproCoreConfig config)' .");
        }
        fileUtils = new FileUtils();
        preferencesUtils = new SharedPreferencesUtils(AdvanproCore.getAppContext(), "AdvanproCore");
        // 检查是否需要删除超过限制次数的过期文件
        deleteOutRetainFile();
        //更新日志文件路径
        updateFileDirPath();

    }

    /**
     * 检查是否需要删除超过限制次数的过期文件
     */
    private void deleteOutRetainFile() {
        String directory = SystemUtils.getExternalFilesDir(AdvanproCore.getAppContext()) + ALogger.LOG_FILE_DIR;
        File dir = new File(directory);
        if (!dir.exists() || !dir.isDirectory())  // 目录不存在
            return;

        List<File> files = Arrays.asList(dir.listFiles());

        Collections.sort(files);//时间升序排序
        int deleteCount = files.size() - logRetainCount;//需要删除数量

        //遍历并删除文件
        for (int i = 0; i < deleteCount; i++) {
            FileUtils.deleteFolderFile(files.get(i).getPath(), true);
        }
    }

    public static boolean d(String name, Object... keyValues) {
        return ALogger.getInstance().log(TYPE_DEBUG, name, keyValues);
    }

    public static boolean i(String name, Object... keyValues) {
        return ALogger.getInstance().log(TYPE_INFO, name, keyValues);
    }

    public static boolean e(String name, Object... keyValues) {
        return ALogger.getInstance().log(TYPE_ERROR, name, keyValues);
    }

    public boolean log(String type, String name, Object... keyValues) {
        StringBuffer sb = new StringBuffer();
        if (keyValues != null && keyValues.length > 0) {
            for (int i = 0; i < keyValues.length; i += 2) {
                if (sb.length() > 0) {
                    sb.append("&");
                }
                if (i + 1 < keyValues.length) {
                    sb.append(keyValues[i] + "=" + keyValues[i + 1]);
                } else {
                    sb.append(keyValues[i]);
                }
            }
        }
        write(type, name, sb.toString());
        return true;
    }

    public boolean log(String type, String name, Map<String, String> keyValues) {
        StringBuffer sb = new StringBuffer();
        if (keyValues != null && keyValues.size() > 0) {
            Set<Map.Entry<String, String>> entries = keyValues.entrySet();
            for (Map.Entry<String, String> e : entries) {
                if (sb.length() > 0) {
                    sb.append("&");
                }
                sb.append(e.getKey() + "=" + e.getValue());
            }
        }
        write(type, name, sb.toString());
        return true;
    }


    /**
     * 更新日志文件目录
     */
    private void updateFileDirPath() {
        if (timestamp == 0) {
            //初始化，从本地sp拿
            timestamp = preferencesUtils.getLong(SP_KEY_LOG_TIMESTAMP, 0);
            lineCount = preferencesUtils.getInt(SP_KEY_LOG_LINE_COUNT, 0);
        }

        //获取当前日期的0点整
        long startOfDay = DateUtils.getStartOfDay(System.currentTimeMillis());
        synchronized (lock) {
            if (timestamp != startOfDay) {
                //如果更换日期
                // 更新行号
                lineCount = 0;
            }
            timestamp = startOfDay;

            //保存日志目录信息，行号信息等
            preferencesUtils.putLong(SP_KEY_LOG_TIMESTAMP, timestamp);
            preferencesUtils.putInt(SP_KEY_LOG_LINE_COUNT, lineCount);
        }
        //初始化日志文件路径
        fileDirPath = SystemUtils.getExternalFilesDir(AdvanproCore.getAppContext())
                + LOG_FILE_DIR + "/" + DateUtils.formatDate(timestamp, "yyyyMMdd");

        //开启定时器，明天0点整，定时更新日志文件目录
        UIUtils.getUIHandler().postAtTime(new Runnable() {
            @Override
            public void run() {
                updateFileDirPath();
            }
        }, DateUtils.getNextDay(startOfDay));
    }


    //写入一行日志
    private boolean write(String type, String name, String value) {
        String time = DateUtils.formatDateToUTC(new Date());
        String data = String.format("%s\t%s\t%s\t%s\n", time, type, name, value);

        //如果是调试模式，打印调试日志
        if (AdvanproCore.isDebug()) {
            switch (type) {
                case TYPE_DEBUG:
                    Log.d(TAG, data);
                    break;
                case TYPE_INFO:
                    Log.i(TAG, data);
                    break;
                case TYPE_ERROR:
                default:
                    android.util.Log.e(TAG, data);
                    break;
            }
        }

        if (!AdvanproCore.isDebug() && type.equals(TYPE_DEBUG)) {
            // 如果 状态模式debug = false ，并且类型 type = TYPE_DEBUG
            // 直接返回，保存到文件
            return true;
        }

        synchronized (lock) {
            //日志行数加自增，并且保存行号信息
            lineCount++;
            preferencesUtils.putInt(SP_KEY_LOG_LINE_COUNT, lineCount);
        }

        //拼接得到日志文件路径
        String fileName = getFileName();
        String path = fileDirPath + "/" + fileName;
        return fileUtils.write(data, path);
    }

    /**
     * 获取日志文件名称
     * 格式：android-timestamp-num.log
     *
     * @return 日志文件名称
     */
    private String getFileName() {
        //platform-timestamp-num.log
        int num = (lineCount / maxLineCount) + 1;
        String fileName = String.format("android-%d-%d.log", timestamp, num);
        return fileName;
    }

    public static boolean e(Throwable ex) {
        return e(ex, null);
    }

    /**
     * 保存错误信息到文件中
     *
     * @param ex
     * @return 返回文件名称, 便于将文件传送到服务器
     */
    public static boolean e(Throwable ex, Map<String, String> infos) {
        StringBuffer sb = new StringBuffer();
        if (infos != null) {
            for (Map.Entry<String, String> entry : infos.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                sb.append(key + "=" + value + "\n");
            }
        }

        Writer writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        ex.printStackTrace(printWriter);
        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        printWriter.close();
        String result = writer.toString();
        sb.append(result);

        return ALogger.getInstance().write(TYPE_ERROR, "throwable", sb.toString());

    }
}
