package hk.advanpro.android.core.raw;

import java.io.File;
import java.util.Date;

import hk.advanpro.android.core.AdvanproCore;
import hk.advanpro.android.core.utils.DateUtils;
import hk.advanpro.android.core.utils.FileUtils;
import hk.advanpro.android.core.utils.SystemUtils;

/**
 * 原始数据类，负责记录原始数据
 * Created by jalin on 2017/12/7.
 */

public class RawDataFile {
    public static final String TYPE_R1 = "r1";
    public static final String TYPE_R2 = "r2";

    public static final String RAW_FILE_DIR = "/raw";
    private FileUtils fileUtils;
    private String filePath;

    private final String type;
    private final String name;
    private final String userId;
    private final Date fileTime;


    /**
     * 获取按天保存的原始文件
     * 按天保存的文件，当文件一段时间未修改并且不是今天时，上传数据
     *
     * @param name   数据类型 如：HGaitLog,StepLog
     * @param time   时间
     * @param userId 用户ID
     * @return RawDataFile
     */
    public static RawDataFile getR1(String name, String userId, Date time) {
        if (name == null || name.isEmpty() || userId == null || userId.isEmpty() || time == null)
            return null;


        return new RawDataFile(TYPE_R1, name, userId, time);
    }

    /**
     * 获取按次保存的原始文件
     * 按次保存的文件，当文件一段时间未修改时，上传数据
     *
     * @param name   数据类型,如：RunLog
     * @param time   时间
     * @param userId 用户ID
     * @return RawDataFile
     */
    public static RawDataFile getR2(String name, String userId, Date time) {
        if (name == null || name.isEmpty() || userId == null || userId.isEmpty() || time == null)
            return null;

        return new RawDataFile(TYPE_R2, name, userId, time);
    }


    private RawDataFile(String type, String name, String userId, Date time) {
        this.type = type;
        this.name = name;
        this.userId = userId;
        this.fileTime = time;


        filePath = formatFilePath(type, name, userId, time);

        fileUtils = new FileUtils();
    }

    /**
     * 格式化 文件路径
     * 路径格式：Raw/userId/name-timestamp.[r1|r2]
     *
     * @param type   类型 如 TYPE_R1， TYPE_R2
     * @param name   名称，如 :StepLog
     * @param userId 用户ID
     * @param time   文件时间
     * @return 格式化
     */
    protected String formatFilePath(String type, String name, String userId, Date time) {
        //拼接文件名
        /// Raw/userId/name-timestamp.[r1|r2]
        String fileName = name + "-" + String.valueOf(time.getTime()) + "." + type;

        //拼接文件地址
        String dir = SystemUtils.getExternalFilesDir(AdvanproCore.getAppContext()) + RAW_FILE_DIR + "/" + userId;
        File folder = new File(dir);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        String filePath = dir + "/" + fileName;
        return filePath;
    }

    public boolean put(Date time, String... data) {
        return log(time, data);
    }

    public boolean put(String... data) {
        return log(data);
    }

    /**
     * 记录原始数据
     *
     * @param data 原始数据
     * @return 是否写入成功
     */
    public boolean log(String... data) {
        return log(new Date(), data);
    }

    /**
     * 记录原始数据，指定时间
     *
     * @param time 记录时间
     * @param data 原始数据
     * @return 是否写入成功
     */
    public boolean log(Date time, String... data) {
        StringBuffer sb = new StringBuffer();
        //时间标签
        sb.append(DateUtils.formatDateToUTC(time));
        for (String s : data) {
            sb.append("\t" + s);
        }
        //换行符
        sb.append("\n");
        return fileUtils.write(sb.toString(), filePath);
    }

    /**
     * 文件名称
     *
     * @return 文件名称
     */
    public String getFileName() {
        return fileUtils.getAbsolutePath();
    }

    /**
     * 文件路径
     *
     * @return 文件路径
     */
    public String getAbsolutePath() {
        return fileUtils.getAbsolutePath();
    }
}
